//**********************************************************************************************************************
//
//  FMColorCorrection.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2011-2015 by IMAGINE GbR. All rights reserved.
//	Abstract:	Applies color correction and a rect mask
//
//**********************************************************************************************************************


uniform sampler2DRect uTexture;

uniform mat4 uColorCorrectionMatrix;
uniform vec4 uGamma;
uniform vec4 uBlackLevelM;
uniform vec4 uBlackLevelT;
uniform vec4 uMixColor;
uniform float uOpacity;


//----------------------------------------------------------------------------------------------------------------------


void main()
{
	float opacity = uOpacity;

	// Get pixel color from texture...
	
	vec2 xy = gl_TexCoord[0].xy;
	vec4 color = texture2DRect(uTexture,xy);
	float alpha = color.a * opacity;
	
	// Apply color correction (tint and saturation via matrix), gamma via pow function...
	
	color = uColorCorrectionMatrix * color;
	color = clamp(color,0.0,1.0);

	// Correct black level (linear equation that boost contrast)...
	
	color = (uBlackLevelM * color) + uBlackLevelT;
	color = clamp(color,0.0,1.0);
	
	// Apply gamma via pow function...
	
	color.rgb = pow(color.rgb,uGamma.rgb);
	
    // Mix with an optional color...
    
    color.rgb = (1.0-uMixColor.a) * color.rgb + uMixColor.a * uMixColor.rgb;
	
	// Pre-multiply the color with pixel alpha and global texture opacity (because all drawing is
	// done with pre-multiplied blending GL_ONE,GL_ONE_MINUS_SRC_ALPHA)...
	
	color.rgb *= opacity;
	color.a = alpha;
	
	// Return corrected color...
	
	gl_FragColor = color;
}


//----------------------------------------------------------------------------------------------------------------------
