//**********************************************************************************************************************
//
//  FMEngineWipeAngularTransition.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2008-2011 by IMAGINE GbR. All rights reserved.
//	Abstract:	Implements a angular wipe transition
//
//**********************************************************************************************************************


uniform sampler2DRect uOutgoingTexture;
uniform sampler2DRect uIncomingTexture;

uniform vec3 uEdge1;
uniform vec3 uEdge2;
uniform float uLeftMixOffset;
uniform float uLeftMixFactor;
uniform float uRightMixOffset;
uniform float uRightMixFactor;

varying vec3 vLocation;


//----------------------------------------------------------------------------------------------------------------------


void main()
{
    // Get both textures...
    
    vec2 xy = gl_TexCoord[0].xy;
	vec4 outgoingColor = texture2DRect(uOutgoingTexture,xy);
	vec4 incomingColor = texture2DRect(uIncomingTexture,xy); 

    // Create the cross product of our pixel vector (vLocation) and the two edge vectors. The result are two 
    // perpendicular vectors, which are either pointing into or out of the stage. The sign of both z values 
    // determines whether we are in the incoming or outgoing sector... 
    
    vec3 v1 = normalize(cross(uEdge1,vLocation));
    vec3 v2 = normalize(cross(uEdge2,vLocation));

    float sign = v1.z * v2.z;
    float mix = 0.5 * (sign + 1.0);
    
    // The left and right half planes need different handling. Without this special handling the "clock effect" 
    // would just be mirrored, and we do not want that. For this reason we'll have to suppress the mirroring
    // by clamping the mix values differently (according to hald plane and fraction)...
    
    if (vLocation.x < 0.0) 
    {
        mix = uLeftMixOffset + uLeftMixFactor * mix;
    }
    else  
    {
        mix = uRightMixOffset + uRightMixFactor * mix;
    }
    
    // Mix incoming and outgoing pixels according to the z-direction...
    
    gl_FragColor = mix(outgoingColor,incomingColor,mix);
}


//----------------------------------------------------------------------------------------------------------------------


