//**********************************************************************************************************************
//
//  FMEngineWipeCircularTransition.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2008-2011 by IMAGINE GbR. All rights reserved.
//	Abstract:	Implements a circular wipe transition
//
//**********************************************************************************************************************


uniform sampler2DRect uOutgoingTexture;
uniform sampler2DRect uIncomingTexture;

uniform float uRadius1;
uniform float uRadius2;
uniform float uMixOffset;
uniform float uMixFactor;

varying vec2 vLocation;


//----------------------------------------------------------------------------------------------------------------------


void main()
{
    // Get outgoing and incoming textures...
    
    vec2 xy = gl_TexCoord[0].xy;
	vec4 outgoingColor = texture2DRect(uOutgoingTexture,xy);
	vec4 incomingColor = texture2DRect(uIncomingTexture,xy); 

    // Calculate distance r from center of stage...
    
    float r = length(vLocation);
    
    // Calculate mixing factor and return appropriate pixel color...
    
    float mix = smoothstep(uRadius1,uRadius2,r);
    mix = uMixOffset + uMixFactor * mix;
    gl_FragColor = mix(incomingColor,outgoingColor,mix);
}


//----------------------------------------------------------------------------------------------------------------------


