//**********************************************************************************************************************
//
//  FMBorderRect.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2013-2016 by IMAGINE GbR. All rights reserved.
//	Abstract:	Draws a rectangular stroke with specified width
//
//**********************************************************************************************************************


uniform vec4 uColor;				// Stroke color
uniform vec2 uDim1;					// Inner edge - 0.5px
uniform vec2 uDim2;					// Inner edge + 0.5px
uniform vec2 uDim3;					// Outer edge - 0.5px
uniform vec2 uDim4;					// Outer edge + 0.5px

varying vec2 vLocation;				// Stage coordinates relative to center of border


//----------------------------------------------------------------------------------------------------------------------


void main()
{
	// Create a mask for the outer edge of the rectangular stroke...
	
    float outerEdgeLeft		= smoothstep(-uDim4.x,-uDim3.x,vLocation.x);
    float outerEdgeRight	= 1.0 - smoothstep(uDim3.x,uDim4.x,vLocation.x);
    float outerEdgeTop		= 1.0 - smoothstep(uDim3.y,uDim4.y,vLocation.y);
    float outerEdgeBottom	= smoothstep(-uDim4.y,-uDim3.y,vLocation.y);
	float outerEdge			= outerEdgeLeft * outerEdgeRight * outerEdgeTop * outerEdgeBottom;

	// Create a mask for the inner edge of the rectangular stroke...
	
    float innerEdgeLeft		= smoothstep(-uDim2.x,-uDim1.x,vLocation.x);
    float innerEdgeRight	= 1.0 - smoothstep(uDim1.x,uDim2.x,vLocation.x);
    float innerEdgeTop		= 1.0 - smoothstep(uDim1.y,uDim2.y,vLocation.y);
    float innerEdgeBottom	= smoothstep(-uDim2.y,-uDim1.y,vLocation.y);
	float innerEdge			= innerEdgeLeft * innerEdgeRight * innerEdgeTop * innerEdgeBottom;

	// Subtract the inner from the outer area, to create the border area. Then apply the color...
	
	float alpha = outerEdge - innerEdge;
	gl_FragColor = uColor.rgba * alpha;
}


//----------------------------------------------------------------------------------------------------------------------
