//**********************************************************************************************************************
//
//  FMBorderOval.fs
//
//  Author:		Peter Baumgartner, peter@baumgartner.com
//  Copyright:	©2013 by IMAGINE GbR. All rights reserved.
//	Abstract:	Draws a elliptical stroke with specified width
//
//**********************************************************************************************************************


uniform vec4 uColor;			// Stroke color
uniform vec2 uDim1;				// Inner edge - 0.5px
uniform vec2 uDim2;				// Inner edge + 0.5px
uniform vec2 uDim3;				// Outer edge - 0.5px
uniform vec2 uDim4;				// Outer edge + 0.5px

varying vec2 vLocation;			// Stage coordinates relative to center of border
const vec4 cExp1 = vec4(2.0);
const vec2 cExp2 = vec2(-0.5);


//----------------------------------------------------------------------------------------------------------------------


// This code was written by Maximillian Christ. Some useful info can be found at:
// http://de.wikipedia.org/wiki/Superellipse
// http://www.yaldex.com/open-gl/ch12lev1sec4.html

float shape(vec2 pos,vec2 innerDim,vec2 outerDim)
{
	vec2 abspos = abs(pos);
	
	vec4 ioxy = vec4(abspos.xy * innerDim.yx, abspos.xy * outerDim.yx);
	vec4 p1 = pow(ioxy,cExp1);
	vec2 iop = p1.xz + p1.yw;
	vec2 iox = vec2(innerDim.x,outerDim.x);
	vec2 ioy = vec2(innerDim.y,outerDim.y);
	vec2 io = iox * ioy * pow(iop,cExp2);
	
	return 1.0 - smoothstep(io.x, io.y, 1.0);
}


//----------------------------------------------------------------------------------------------------------------------


void main()
{
	// Create a mask for the outer edge of the rectangular stroke...

	float outerEdge = shape(vLocation.xy,uDim3,uDim4);

	// Create a mask for the inner edge of the rectangular stroke...
	
	float innerEdge = shape(vLocation.xy,uDim1,uDim2);

	// Subtract the inner from the outer area, to create the border area. Then apply the color...
	
	float alpha = outerEdge - innerEdge;
	gl_FragColor = uColor.rgba * alpha;
}


//----------------------------------------------------------------------------------------------------------------------
